/***************************************************************************
 *   Copyright (C) 2007-2012 by Tobias Schwarz   *
 *   mail@tobias-schwarz.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include "dbDlg.h"
#include "kembossWindowClass.h"

dbDlg::dbDlg(kemboss *parent, kembossSettings *fksettings) : QMainWindow(parent)
{
   setupUi(this);
   setAttribute(Qt::WA_DeleteOnClose);
   mainKembossWindow = parent;
   ksettings = fksettings;
   readSettings();
   runEmbossApp = new krunEmbossApp(this, fksettings);

   connect(DonePushButton, SIGNAL(clicked()), this, SLOT(done()));
   connect(HelpPushButton, SIGNAL(clicked()), this, SLOT(helpRequested()));

   connect(ProteinCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(NucleicCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(SequenceCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(FeatureCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(TextCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(TaxonomyCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(ResourceCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(AssemblyCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(OboCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(MethodsCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(FieldsCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(DefinedCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(ReleaseCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(TypeCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(IdCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(QueryCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(AllCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));
   connect(CommentCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableDatabases()));

   connect(ProteinCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(NucleicCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(SequenceCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(ProtfeaturesCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(NucfeaturesCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(FeatureCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(OboCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(ResourceCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(TaxonomyCheckBox_3, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(AssemblyCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(TextCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(HTMLCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(XMLCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(UnknownCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(MethodsCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(CountCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(CachefileCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(URLCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(FieldsCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(DefinedCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(ServerversionCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(ScopeCheckBox, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(TypeCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(IdCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(QueryCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(AllCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));
   connect(CommentCheckBox_2, SIGNAL(stateChanged(int)), this, SLOT(getAvailableServer()));

   getAvailableDatabases();
   getAvailableServer();
}

void dbDlg::done()
{
   writeSettings();
   close();
}

void dbDlg::helpRequested()
{
   QMessageBox::information(this, "kemboss", tr("Sorry, help is still missing here."));
}

void dbDlg::closeEvent(QCloseEvent *event)
{
   runEmbossApp->~krunEmbossApp();
   event->accept();
}

void dbDlg::readSettings()
{
   QSettings settings(QSettings::UserScope, "kemboss.org", "dbDlg");
   QPoint pos = settings.value("db_pos").toPoint();
   QSize size = settings.value("db_size").toSize();
   if (!size.isEmpty()) resize(size);
   if (!pos.isNull()) move(pos);

   /*Available databases */
   ProteinCheckBox->setChecked(settings.value("Protein", true).toBool());
   NucleicCheckBox->setChecked(settings.value("Nucleic", true).toBool());
   SequenceCheckBox->setChecked(settings.value("Sequence", true).toBool());
   FeatureCheckBox->setChecked(settings.value("Feature", true).toBool());
   TextCheckBox->setChecked(settings.value("Text", true).toBool());
   TaxonomyCheckBox_2->setChecked(settings.value("Taxonomy_2", true).toBool());
   ResourceCheckBox->setChecked(settings.value("Resource", true).toBool());
   AssemblyCheckBox->setChecked(settings.value("Assembly", true).toBool());
   OboCheckBox->setChecked(settings.value("Obo", true).toBool());
   MethodsCheckBox->setChecked(settings.value("Methods", true).toBool());
   FieldsCheckBox->setChecked(settings.value("Fields", true).toBool());
   DefinedCheckBox->setChecked(settings.value("Defined", true).toBool());
   ReleaseCheckBox->setChecked(settings.value("Release", true).toBool());
   TypeCheckBox->setChecked(settings.value("Type", true).toBool());
   IdCheckBox->setChecked(settings.value("Id_2", true).toBool());
   QueryCheckBox->setChecked(settings.value("Query", true).toBool());
   AllCheckBox->setChecked(settings.value("All", true).toBool());
   CommentCheckBox->setChecked(settings.value("Comment", true).toBool());

   DbtellFullCheckBox->setChecked(settings.value("dbtell_full", false).toBool());

   /* Extract database */
   DatabaseTypeComboBox->setCurrentIndex(settings.value("dbindex_prog").toInt());
   IDCheckBox->setChecked(settings.value("id", false).toBool());
   AccNoCheckBox->setChecked(settings.value("acc_no", false).toBool());
   SVAndGICheckBox->setChecked(settings.value("sv_and_gi", false).toBool());
   DescriptionCheckBox->setChecked(settings.value("description", false).toBool());
   KeywordsCheckBox->setChecked(settings.value("keywords", false).toBool());
   TaxonomyCheckBox->setChecked(settings.value("taxonomy", false).toBool());
   NameCheckBox->setChecked(settings.value("nam", false).toBool());
   ParentCheckBox->setChecked(settings.value("isa", false).toBool());
   NamespaceCheckBox->setChecked(settings.value("ns", false).toBool());
   HasInputCheckBox->setChecked(settings.value("hasin", false).toBool());
   HasOutputCheckBox->setChecked(settings.value("hasout", false).toBool());
   IsIdentifierOfCheckBox->setChecked(settings.value("isid", false).toBool());
   IsFormatOfCheckBox->setChecked(settings.value("isfmt", false).toBool());
   URLCheckBox->setChecked(settings.value("url", false).toBool());
   CategoryNameCheckBox->setChecked(settings.value("cat", false).toBool());
   TaxonIDCheckBox->setChecked(settings.value("taxid", false).toBool());
   EdatCheckBox->setChecked(settings.value("edat", false).toBool());
   EfmtCheckBox->setChecked(settings.value("efmt", false).toBool());
   EidCheckBox->setChecked(settings.value("eid", false).toBool());
   EtpcCheckBox->setChecked(settings.value("etpc", false).toBool());
   XrefCheckBox->setChecked(settings.value("xref", false).toBool());
   QoutCheckBox->setChecked(settings.value("qout", false).toBool());
   QfmtCheckBox->setChecked(settings.value("qfmt", false).toBool());
   QinCheckBox->setChecked(settings.value("qin", false).toBool());
   QurlCheckBox->setChecked(settings.value("qurl", false).toBool());
   RestCheckBox->setChecked(settings.value("rest", false).toBool());
   SOAPCheckBox->setChecked(settings.value("soap", false).toBool());
   TaxCheckBox->setChecked(settings.value("tax", false).toBool());
   RnkCheckBox->setChecked(settings.value("rnk", false).toBool());
   UpCheckBox->setChecked(settings.value("up", false).toBool());
   GcCheckBox->setChecked(settings.value("gc", false).toBool());
   MgcCheckBox->setChecked(settings.value("mgc", false).toBool());
   DatabaseNameLineEdit->setText(settings.value("db_name").toString());
   StandardDbNameLineEdit->setText(settings.value("standard_db_name").toString());
   ResourceNameLineEdit->setText(settings.value("resource_name").toString());
   FormatComboBox->setCurrentIndex(settings.value("id_format").toInt());
   ReleaseLineEdit->setText(settings.value("release", "0.0").toString());
   IndexDateLineEdit->setText(QDate::currentDate().toString("dd/MM/yy"));
   BlastIndexVersionComboBox->setCurrentIndex(settings.value("blast_index_version").toInt());
   MinOccurencesSpinBox->setValue(settings.value("min_occurences").toInt());
   MaxOccurencesSpinBox->setValue(settings.value("max_occurences").toInt());
   UseFASTASourceFileCheckBox->setChecked(settings.value("use_fasta_source", false).toBool());
   StatisticsCheckBox->setChecked(settings.value("statistics", false).toBool());
   CompressCheckBox->setChecked(settings.value("compress", true).toBool());
   FullreportCheckBox->setChecked(settings.value("fullreport", false).toBool());
   UseSystemSortUtilityCheckBox->setChecked(settings.value("use_system_sort", true).toBool());
   CleanUpTemporaryFilesCheckBox->setChecked(settings.value("clean_up", true).toBool());
   MaximumIndexLengthSpinBox->setValue(settings.value("max_index_length").toInt());
   SortOptionsLineEdit->setText(settings.value("sort_options", "-T . -k 1,1").toString());
   DatabaseDirectoryLineEdit->setText(settings.value("db_dir").toString());
   DatabaseFilenameLineEdit->setText(settings.value("db_filename", "*.dat").toString());
   WildcardFilenamesToExcludeLineEdit->setText(settings.value("wildcard_filenames").toString());
   IndexDirectoryLineEdit->setText(settings.value("index_dir", ".").toString());
   LogOutputFileLineEdit->setText(settings.value("log_output_file", "dbi.log").toString());

   /*Available server */
   ProteinCheckBox_2->setChecked(settings.value("Protein_2", true).toBool());
   NucleicCheckBox_2->setChecked(settings.value("Nucleic_2", true).toBool());
   SequenceCheckBox_2->setChecked(settings.value("Sequence_2", true).toBool());
   ProtfeaturesCheckBox->setChecked(settings.value("Protfeatures", true).toBool());
   NucfeaturesCheckBox->setChecked(settings.value("Nucfeatures", true).toBool());
   FeatureCheckBox_2->setChecked(settings.value("Feature_2", true).toBool());
   OboCheckBox_2->setChecked(settings.value("Obo_2", true).toBool());
   ResourceCheckBox_2->setChecked(settings.value("Resource_2", true).toBool());
   TaxonomyCheckBox_3->setChecked(settings.value("Taxonomy_3", true).toBool());
   AssemblyCheckBox_2->setChecked(settings.value("Assembly_2", true).toBool());
   TextCheckBox_2->setChecked(settings.value("Text_2", true).toBool());
   HTMLCheckBox->setChecked(settings.value("HTML", true).toBool());
   XMLCheckBox->setChecked(settings.value("XML", true).toBool());
   UnknownCheckBox->setChecked(settings.value("Unknown", true).toBool());
   MethodsCheckBox_2->setChecked(settings.value("Methods_2", true).toBool());
   CountCheckBox->setChecked(settings.value("Count", true).toBool());
   CachefileCheckBox->setChecked(settings.value("Cachefile", true).toBool());
   URLCheckBox_2->setChecked(settings.value("URL_2", true).toBool());
   FieldsCheckBox_2->setChecked(settings.value("Fields_2", true).toBool());
   DefinedCheckBox_2->setChecked(settings.value("Defined_2", true).toBool());
   ServerversionCheckBox->setChecked(settings.value("Serverversion", true).toBool());
   ScopeCheckBox->setChecked(settings.value("Scope", true).toBool());
   TypeCheckBox_2->setChecked(settings.value("Type_2", true).toBool());
   IdCheckBox_2->setChecked(settings.value("Id_3", true).toBool());
   QueryCheckBox_2->setChecked(settings.value("Query_2", true).toBool());
   AllCheckBox_2->setChecked(settings.value("All_2", true).toBool());
   CommentCheckBox_2->setChecked(settings.value("Comment_2", true).toBool());

   ServertellFullCheckBox->setChecked(settings.value("servertell_full", false).toBool());

   /* Generate Cachefiles */
   CachefileLineEdit->setText(settings.value("cachefile","server.$(servername)").toString());
   OutfileLineEdit->setText(settings.value("outfile").toString());
   HostLineEdit->setText(settings.value("host","www.dasregistry.org").toString());
   PathLineEdit->setText(settings.value("path","/das").toString());
   PortSpinBox->setValue(settings.value("port",80).toInt());
   ServernameLineEdit->setText(settings.value("servername","das").toString());
   SequencesourcesonlyCheckBox->setChecked(settings.value("sequencesourcesonly", false).toBool());
}

void dbDlg::writeSettings()
{
   QSettings settings(QSettings::UserScope, "kemboss.org", "dbDlg");
   settings.setValue("db_pos", pos());
   settings.setValue("db_size", size());

   /* Available databases */
   settings.setValue("Protein", ProteinCheckBox->checkState());
   settings.setValue("Nucleic", NucleicCheckBox->checkState());
   settings.setValue("Sequence", SequenceCheckBox->checkState());
   settings.setValue("Feature", FeatureCheckBox->checkState());
   settings.setValue("Text", TextCheckBox->checkState());
   settings.setValue("Taxonomy_2", TaxonomyCheckBox_2->checkState());
   settings.setValue("Resource", ResourceCheckBox->checkState());
   settings.setValue("Assembly", AssemblyCheckBox->checkState());
   settings.setValue("Obo", OboCheckBox->checkState());
   settings.setValue("Methods", MethodsCheckBox->checkState());
   settings.setValue("Fields", FieldsCheckBox->checkState());
   settings.setValue("Defined", DefinedCheckBox->checkState());
   settings.setValue("Release", ReleaseCheckBox->checkState());
   settings.setValue("Type", TypeCheckBox->checkState());
   settings.setValue("Id_2", IdCheckBox->checkState());
   settings.setValue("Query", QueryCheckBox->checkState());
   settings.setValue("All", AllCheckBox->checkState());
   settings.setValue("Comment", CommentCheckBox->checkState());

   settings.setValue("dbtell_full", DbtellFullCheckBox->checkState());

   /* Index database */
   settings.setValue("dbindex_prog", DatabaseTypeComboBox->currentIndex());
   settings.setValue("id", IDCheckBox->checkState());
   settings.setValue("acc_no", AccNoCheckBox->checkState());
   settings.setValue("sv_and_gi", SVAndGICheckBox->checkState());
   settings.setValue("description", DescriptionCheckBox->checkState());
   settings.setValue("keywords", KeywordsCheckBox->checkState());
   settings.setValue("taxonomy", TaxonomyCheckBox->checkState());
   settings.setValue("nam", NameCheckBox->checkState());
   settings.setValue("isa", ParentCheckBox->checkState());
   settings.setValue("ns", NamespaceCheckBox->checkState());
   settings.setValue("hasin", HasInputCheckBox->checkState());
   settings.setValue("hasout", HasOutputCheckBox->checkState());
   settings.setValue("isid", IsIdentifierOfCheckBox->checkState());
   settings.setValue("isfmt", IsFormatOfCheckBox->checkState());
   settings.setValue("url", URLCheckBox->checkState());
   settings.setValue("cat", CategoryNameCheckBox->checkState());
   settings.setValue("taxid", TaxonIDCheckBox->checkState());
   settings.setValue("edat", EdatCheckBox->checkState());
   settings.setValue("efmt", EfmtCheckBox->checkState());
   settings.setValue("eid", EidCheckBox->checkState());
   settings.setValue("etpc", EtpcCheckBox->checkState());
   settings.setValue("xref", XrefCheckBox->checkState());
   settings.setValue("qout", QoutCheckBox->checkState());
   settings.setValue("qfmt", QfmtCheckBox->checkState());
   settings.setValue("qin", QinCheckBox->checkState());
   settings.setValue("qurl", QurlCheckBox->checkState());
   settings.setValue("rest", RestCheckBox->checkState());
   settings.setValue("soap", SOAPCheckBox->checkState());
   settings.setValue("tax", TaxCheckBox->checkState());
   settings.setValue("rnk", RnkCheckBox->checkState());
   settings.setValue("up", UpCheckBox->checkState());
   settings.setValue("gc", GcCheckBox->checkState());
   settings.setValue("mgc", MgcCheckBox->checkState());
   settings.setValue("db_name", DatabaseNameLineEdit->text());
   settings.setValue("standard_db_name", StandardDbNameLineEdit->text());
   settings.setValue("resource_name", ResourceNameLineEdit->text());
   settings.setValue("id_format", FormatComboBox->currentIndex());
   settings.setValue("release", ReleaseLineEdit->text());
   settings.setValue("blast_index_version", BlastIndexVersionComboBox->currentIndex());
   settings.setValue("min_occurences", MinOccurencesSpinBox->value());
   settings.setValue("max_occurences", MaxOccurencesSpinBox->value());
   settings.setValue("use_fasta_source", UseFASTASourceFileCheckBox->checkState());
   settings.setValue("statistics", StatisticsCheckBox->checkState());
   settings.setValue("compress", CompressCheckBox->checkState());
   settings.setValue("fullreport", FullreportCheckBox->checkState());
   settings.setValue("use_system_sort", UseSystemSortUtilityCheckBox->checkState());
   settings.setValue("clean_up", CleanUpTemporaryFilesCheckBox->checkState());
   settings.setValue("max_index_length", MaximumIndexLengthSpinBox->value());
   settings.setValue("sort_options", SortOptionsLineEdit->text());
   settings.setValue("db_dir", DatabaseDirectoryLineEdit->text());
   settings.setValue("db_filename", DatabaseFilenameLineEdit->text());
   settings.setValue("wildcard_filenames", WildcardFilenamesToExcludeLineEdit->text());
   settings.setValue("index_dir", IndexDirectoryLineEdit->text());
   settings.setValue("log_output_file", LogOutputFileLineEdit->text());

   /* Available server */
   settings.setValue("servertell_full", ServertellFullCheckBox->checkState());

   settings.setValue("Protein_2", ProteinCheckBox_2->text());
   settings.setValue("Nucleic_2", NucleicCheckBox_2->text());
   settings.setValue("Sequence_2", SequenceCheckBox_2->text());
   settings.setValue("Protfeatures", ProtfeaturesCheckBox->text());
   settings.setValue("Nucfeatures", NucfeaturesCheckBox->text());
   settings.setValue("Feature_2", FeatureCheckBox_2->text());
   settings.setValue("Obo_2", OboCheckBox_2->text());
   settings.setValue("Resource_2", ResourceCheckBox_2->text());
   settings.setValue("Taxonomy_3", TaxonomyCheckBox_3->text());
   settings.setValue("Assembly_2", AssemblyCheckBox_2->text());
   settings.setValue("Text_2", TextCheckBox_2->text());
   settings.setValue("HTML", HTMLCheckBox->text());
   settings.setValue("XML", XMLCheckBox->text());
   settings.setValue("Unknown", UnknownCheckBox->text());
   settings.setValue("Methods_2", MethodsCheckBox_2->text());
   settings.setValue("Count", CountCheckBox->text());
   settings.setValue("Cachefile", CachefileCheckBox->text());
   settings.setValue("URL_2", URLCheckBox_2->text());
   settings.setValue("Fields_2", FieldsCheckBox_2->text());
   settings.setValue("Defined_2", DefinedCheckBox_2->text());
   settings.setValue("Serverversion", ServerversionCheckBox->text());
   settings.setValue("Scope", ScopeCheckBox->text());
   settings.setValue("Type_2", TypeCheckBox_2->text());
   settings.setValue("Id_3", IdCheckBox_2->text());
   settings.setValue("Query_2", QueryCheckBox_2->text());
   settings.setValue("All_2", AllCheckBox_2->text());
   settings.setValue("Comment_2", CommentCheckBox_2->text());

   /* Generate Cachefiles */
   settings.setValue("cachefile", CachefileLineEdit->text());
   settings.setValue("outfile", OutfileLineEdit->text());
   settings.setValue("host", HostLineEdit->text());
   settings.setValue("path", PathLineEdit->text());
   settings.setValue("port", PortSpinBox->value());
   settings.setValue("servername", ServernameLineEdit->text());
   settings.setValue("sequencesourcesonly", SequencesourcesonlyCheckBox->checkState());
}

void dbDlg::setExistingDir(const QString &dlgMsg, QLineEdit *lineEdit)
{
   QString dirName = QDir::toNativeSeparators(QFileDialog::getExistingDirectory(this, dlgMsg, ksettings->currentDir(), QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks));
   if (!dirName.isEmpty()) {
      lineEdit->setText(dirName);
   }
}

void dbDlg::setOpenFile(const QString &dlgMsg, QLineEdit *lineEdit)
{
   QString fileName = QFileDialog::getOpenFileName(this, dlgMsg, ksettings->currentDir());
   if (!fileName.isEmpty()) {
      lineEdit->setText(fileName);
   }
}

void dbDlg::setSaveFile(const QString &dlgMsg, QLineEdit *lineEdit)
{
   QString fileName = QFileDialog::getSaveFileName(this, dlgMsg, ksettings->currentDir());
   if (!fileName.isEmpty()) {
      lineEdit->setText(fileName);
   }
}

void dbDlg::extractDbChanged()
{
   REBASEGroupBox->setChecked(false);
   CUTGGroupBox->setChecked(false);
   AAINDEXGroupBox->setChecked(false);
   JASPARGroupBox->setChecked(false);
   PRINTSGroupBox->setChecked(false);
   PROSITEGroupBox->setChecked(false);
   TRANSFACGroupBox->setChecked(false);
   if (database == "rebase")
      REBASEGroupBox->setChecked(true);
   else if (database == "cutg")
      CUTGGroupBox->setChecked(true);
   else if (database == "aaindex")
      AAINDEXGroupBox->setChecked(true);
   else if (database == "pros")
      PROSITEGroupBox->setChecked(true);
   else if (database == "jasp")
      JASPARGroupBox->setChecked(true);
   else if (database == "tf")
      TRANSFACGroupBox->setChecked(true);
   else if (database == "prints")
      PRINTSGroupBox->setChecked(true);
}

void dbDlg::on_OpenHomepagePushButton_clicked()
{
   if (database == "rebase")
      QDesktopServices::openUrl (QUrl("http://rebase.neb.com/"));
   else if (database == "cutg")
      QDesktopServices::openUrl (QUrl("ftp://ftp.ebi.ac.uk/pub/databases/cutg/"));
   else if (database == "aaindex")
      QDesktopServices::openUrl (QUrl("ftp://ftp.genome.ad.jp/pub/db/genomenet/aaindex/"));
   else if (database == "pros")
      QDesktopServices::openUrl (QUrl(""));
   else if (database == "jasp")
      QDesktopServices::openUrl (QUrl("http://jaspar.genereg.net/"));
   else if (database == "tf")
      QDesktopServices::openUrl (QUrl("http://www.gene-regulation.com/pub/databases.html"));
   else if (database == "prints")
      QDesktopServices::openUrl (QUrl("http://www.bioinf.man.ac.uk/dbbrowser/PRINTS/"));
}

void dbDlg::on_ExtractDatabasePushButton_clicked()
{
   runEmbossApp->programConfiguration.clear();
   runEmbossApp->programConfiguration.append("-auto");

   if (database == "rebase") {
      if (WithrefmFileLineEdit->text().isEmpty() || !QFile::exists(WithrefmFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct withrefm file."));
         return;
      }
      if (ProtoFileLineEdit->text().isEmpty() || !QFile::exists(ProtoFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct proto file."));
         return;
      }
      runEmbossApp->programConfiguration.append(WithrefmFileLineEdit->text());
      runEmbossApp->programConfiguration.append(ProtoFileLineEdit->text());
      if (!CalculateEquFileCheckBox->isChecked()) runEmbossApp->programConfiguration.append("-noequivalences");
   } else if (database == "cutg") {
      QDir cutgDir(CUTGDirectoryLineEdit->text());
      if (CUTGDirectoryLineEdit->text().isEmpty() || !cutgDir.exists()) {
         QMessageBox::information(this, "kemboss", tr("Please select correct CUTG data directory."));
         return;
      }
      if (GeneralLogOutputFileLineEdit->text().isEmpty()) {
         QMessageBox::information(this, "kemboss", tr("Please select correct CUTG log output file."));
         return;
      }
      runEmbossApp->programConfiguration.append(CUTGDirectoryLineEdit->text());
      runEmbossApp->programConfiguration.append(GeneralLogOutputFileLineEdit->text());
      if (!CUTGReleaseLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-release "+CUTGReleaseLineEdit->text());
      if (!TypeOfCodonFileLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-wildspec "+TypeOfCodonFileLineEdit->text());
      if (!SpeciesWildcardLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-species "+SpeciesWildcardLineEdit->text());
      if (!SingleOutputFilenameLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-filename "+SingleOutputFilenameLineEdit->text());
      if (IncludeAllRecordsCheckBox->isChecked()) runEmbossApp->programConfiguration.append("-allrecords");
   } else if (database == "aaindex") {
      if (AAINDEXFileLineEdit->text().isEmpty() || !QFile::exists(AAINDEXFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct AAINDEX file."));
         return;
      }
      runEmbossApp->programConfiguration.append(AAINDEXFileLineEdit->text());
   } else if (database == "pros") {
      QDir prosDir(PROSITEDirectoryLineEdit->text());
      if (PROSITEDirectoryLineEdit->text().isEmpty() || !prosDir.exists()) {
         QMessageBox::information(this, "kemboss", tr("Please select the correct PROSITE database directory."));
         return;
      }
      runEmbossApp->programConfiguration.append(PROSITEDirectoryLineEdit->text());
   } else if (database == "jasp") {
      QDir jaspDir(JASPARDirectoryLineEdit->text());
      if (JASPARDirectoryLineEdit->text().isEmpty() || !jaspDir.exists()) {
         QMessageBox::information(this, "kemboss", tr("Please select the correct JASPAR data directory."));
         return;
      }
      runEmbossApp->programConfiguration.append(JASPARDirectoryLineEdit->text());
   } else if (database == "tf") {
      if (TRANSFACFileLineEdit->text().isEmpty() || !QFile::exists(TRANSFACFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct TRANSFAC 'site.dat' file."));
         return;
      }
      runEmbossApp->programConfiguration.append(TRANSFACFileLineEdit->text());
   } else if (database == "prints") {
      if (PRINTSFileLineEdit->text().isEmpty() || !QFile::exists(PRINTSFileLineEdit->text())) {
         QMessageBox::information(this, "kemboss", tr("Please select correct PRINTS database file."));
         return;
      }
      runEmbossApp->programConfiguration.append(PRINTSFileLineEdit->text());
   }
   runEmbossApp->setEmbossApp(database+"extract");
   runEmbossApp->runApp();
}

void dbDlg::getAvailableDatabases() {
   AvailableDatabasesTextEdit->document()->clear();
   QProcess showdb;
   QString prog = ksettings->embossBin()+"showdb";
   if (!ProteinCheckBox->checkState()) prog += " -noprotein";
   if (!NucleicCheckBox->checkState()) prog += " -nonucleic";
   if (!SequenceCheckBox->checkState()) prog += " -nosequence";
   if (!FeatureCheckBox->checkState()) prog += " -nofeature";
   if (!TextCheckBox->checkState()) prog += " -notext";
   if (!TaxonomyCheckBox_2->checkState()) prog += " -notaxonomy";
   if (!ResourceCheckBox->checkState()) prog += " -noresource";
   if (!AssemblyCheckBox->checkState()) prog += " -noassembly";
   if (!OboCheckBox->checkState()) prog += " -noobo";
   if (MethodsCheckBox->checkState()) prog += " -methods";
   if (FieldsCheckBox->checkState()) prog += " -fields";
   if (DefinedCheckBox->checkState()) prog += " -defined";
   if (ReleaseCheckBox->checkState()) prog += " -release";
   if (!TypeCheckBox->checkState()) prog += " -notype";
   if (!IdCheckBox->checkState()) prog += " -noid";
   if (!QueryCheckBox->checkState()) prog += " -noquery";
   if (!AllCheckBox->checkState()) prog += " -noall";
   if (!CommentCheckBox->checkState()) prog += " -nocomment";

   showdb.start(prog);
   if (!showdb.waitForFinished(ksettings->maxExeTime())) return; // waits for program showdb to finish
   QByteArray programOutput = showdb.readAll();
   AvailableDatabasesTextEdit->setText(QString(programOutput));
}

void dbDlg::getAvailableServer() {
   AvailableServerTextEdit->document()->clear();
   QProcess showserver;
   QString prog = ksettings->embossBin()+"showserver -showtypes ";

   if (ProteinCheckBox_2->checkState()) prog += "Protein,";
   if (NucleicCheckBox_2->checkState()) prog += "Nucleic,";
   if (SequenceCheckBox_2->checkState()) prog += "Sequence,";
   if (ProtfeaturesCheckBox->checkState()) prog += "Protfeatures,";
   if (NucfeaturesCheckBox->checkState()) prog += "Nucfeatures,";
   if (FeatureCheckBox_2->checkState()) prog += "Features,";
   if (OboCheckBox_2->checkState()) prog += "Obo,";
   if (ResourceCheckBox_2->checkState()) prog += "Resource,";
   if (TaxonomyCheckBox_3->checkState()) prog += "Taxonomy,";
   if (AssemblyCheckBox_2->checkState()) prog += "Assembly,";
   if (TextCheckBox_2->checkState()) prog += "Text,";
   if (HTMLCheckBox->checkState()) prog += "Html,";
   if (XMLCheckBox->checkState()) prog += "Xml,";
   if (UnknownCheckBox->checkState()) prog += "Unknown,";
   prog.chop(1);

   if (MethodsCheckBox_2->checkState()) prog += " -methods";
   if (!CountCheckBox->checkState()) prog += " -nocount";
   if (!CachefileCheckBox->checkState()) prog += " -nocachefile";
   if (!URLCheckBox_2->checkState()) prog += " -nourl";
   if (FieldsCheckBox_2->checkState()) prog += " -fields";
   if (DefinedCheckBox_2->checkState()) prog += " -defined";
   if (ServerversionCheckBox->checkState()) prog += " -serverversion";
   if (!ScopeCheckBox->checkState()) prog += " -noscope";
   if (!TypeCheckBox_2->checkState()) prog += " -notype";
   if (!IdCheckBox_2->checkState()) prog += " -noid";
   if (!QueryCheckBox_2->checkState()) prog += " -noquery";
   if (!AllCheckBox_2->checkState()) prog += " -noall";
   if (!CommentCheckBox_2->checkState()) prog += " -nocomment";

   showserver.start(prog);
   if (!showserver.waitForFinished(ksettings->maxExeTime())) return; // waits for program showserver to finish
   QByteArray programOutput = showserver.readAll();
   AvailableServerTextEdit->setText(QString(programOutput));
}

void dbDlg::on_MethodComboBox_currentIndexChanged(const QString &text)
{
   if (text == "emboss" || text == "srswww") {
      NamespaceLineEdit->setEnabled(true); NamespaceLabel->setEnabled(true);
      SpecialLineEdit->setEnabled(true); SpecialLabel->setEnabled(true);
      OrganismsLineEdit->setEnabled(true); OrganismsLabel->setEnabled(true);
   } else {
      NamespaceLineEdit->setEnabled(false); NamespaceLabel->setEnabled(false);
      SpecialLineEdit->setEnabled(false); SpecialLabel->setEnabled(false);
      OrganismsLineEdit->setEnabled(false); OrganismsLabel->setEnabled(false);
   }
}

void dbDlg::on_AddDBPushButton_clicked()
{
   if (NameLineEdit->text().isEmpty()) {
      QMessageBox::warning(this, "kemboss", tr("Please, enter a name for the database."));
      return;
   }
   QString fileName;
   if (WriteToComboBox->currentIndex() == 1) { //Write to emboss.default
      if (QFile::exists(ksettings->embossBin()+"emboss.default")) fileName = ksettings->embossBin()+"emboss.default"; //Default on MS Windows
      else if (QFile::exists(ksettings->embossData()+"../emboss.default")) fileName = ksettings->embossData()+"../emboss.default";  //Default on UNIX
      if (fileName.isEmpty()) {
         QMessageBox::warning(this, "kemboss", tr("emboss.default file not found!\nCreating new emboss.default file."));
         #ifdef Q_WS_WIN
            fileName = ksettings->embossBin()+"emboss.default"; //Default on MS Windows
         #else
            fileName = ksettings->embossData()+"../emboss.default";  //Default on UNIX
         #endif
      }
   } else { //Write to .embossrc in users home directory
      fileName = QDir::homePath()+QDir::separator()+".embossrc";
   }
   QFile file(fileName);
   if (!file.open(QFile::Append | QFile::Text)) {
      QMessageBox::warning(this, objectName(),
                   tr("Cannot write file %1:\n%2.")
                   .arg(fileName)
                   .arg(file.errorString()));
      return;
   }
   QTextStream out(&file);
   out << "\n";
   out << "#Database definition created by kemboss\n";
   out << "DB "+NameLineEdit->text()+" [\n";
   out << "   method: \""+MethodComboBox->currentText()+"\"\n";
   out << "   type: \""+TypeComboBox->currentText()+"\"\n";
   out << "   format: \""+FormatComboBox_2->currentText()+"\"\n";

   if (!FieldsPlainTextEdit->toPlainText().isEmpty()) {
       QStringList list = FieldsPlainTextEdit->toPlainText().split("\n");
       for (int i = 0; i < list.size(); ++i)
          out << "   field: \""+list.at(i)+"\"\n";
   }
   if (!ReleaseLineEdit_2->text().isEmpty()) out << "   release: \""+ReleaseLineEdit_2->text()+"\"\n";
   if (SpecialLineEdit->isEnabled() && !SpecialLineEdit->text().isEmpty()) out << "   special: \""+SpecialLineEdit->text()+"\"\n";
   if (NamespaceLineEdit->isEnabled() && !NamespaceLineEdit->text().isEmpty()) out << "   namespace: \""+NamespaceLineEdit->text()+"\"\n";
   if (OrganismsLineEdit->isEnabled() && !OrganismsLineEdit->text().isEmpty()) out << "   organisms: \""+OrganismsLineEdit->text()+"\"\n";
   if (!DBAliasLineEdit->text().isEmpty()) out << "   dbalias: \""+DBAliasLineEdit->text()+"\"\n";
   if (!URLLineEdit->text().isEmpty()) out << "   url: \""+URLLineEdit->text()+"\"\n";
   if (!AppLineEdit->text().isEmpty()) out << "   app: \""+AppLineEdit->text()+"\"\n";
   if (!DirectoryLineEdit->text().isEmpty()) out << "   dir: \""+DirectoryLineEdit->text()+"\"\n";
   if (!FileLineEdit->text().isEmpty()) out << "   file: \""+FileLineEdit->text()+"\"\n";
   if (!ExludeLineEdit->text().isEmpty()) out << "   exclude: \""+ExludeLineEdit->text()+"\"\n";
   if (!IndexDirLineEdit->text().isEmpty()) out << "   indexdir: \""+IndexDirLineEdit->text()+"\"\n";
   if (!CommentLineEdit->text().isEmpty()) out << "   comment: \""+CommentLineEdit->text()+"\"\n";
   if (!IdentifierLineEdit->text().isEmpty()) out << "   identifier: \""+IdentifierLineEdit->text()+"\"\n";
   if (!SequenceLineEdit->text().isEmpty()) out << "   sequence: \""+SequenceLineEdit->text()+"\"\n";
   if (!FilterLineEdit->text().isEmpty()) out << "   filter: \""+FilterLineEdit->text()+"\"\n";
   if (!ReturnLineEdit->text().isEmpty()) out << "   return: \""+ReturnLineEdit->text()+"\"\n";
   out << "]\n";
   getAvailableDatabases();
}

void dbDlg::on_DatabaseTypeComboBox_currentIndexChanged(const QString &text)
{
   FormatComboBox->clear();
   QStringList idItems;
   if (text == "dbiblast") {
      FormatLabel->setText("Sequence type:");
      idItems << "N (nucleic)" << "? (unknown)" << "P (protein)";
   } else if (text == "dbifasta") {
      FormatLabel->setText("ID format:");
      idItems << "simple (>ID)" << "idacc (>ID ACC) or (>ID (ACC))" << "idsv (>ID SV) or (>ID (SV))" << "gcgid (>db:ID)" << "gcgidacc (>db:ID ACC)" << "dbid (>db ID)" << "ncbi (| formats)";
   } else if (text == "dbxfasta") {
       FormatLabel->setText("ID format:");
       idItems << "simple (>ID)" << "idacc (>ID ACC) or (>ID (ACC))" << "gcgid (>db:ID)" << "gcgidacc (>db:ID ACC)" << "dbid (>db ID)" << "ncbi (| formats)";
    } else if (text == "dbiflat") {
     FormatLabel->setText("Entry format:");
     idItems << "EMBL (EMBL)" << "SWISS (Swiss-Prot, SpTrEMBL, TrEMBLnew)" << "GB (Genbank, DDBJ)" << "REFSEQ (Refseq)";
   } else if (text == "dbxflat") {
      FormatLabel->setText("Entry format:");
      idItems << "EMBL (EMBL)" << "SWISS (Swiss-Prot, SpTrEMBL, TrEMBLnew)" << "GB (Genbank, DDBJ)" << "REFSEQ (Refseq)" << "FASTQ (Fastq files)";
   } else if (text == "dbigcg" || text == "dbxgcg") {
      FormatLabel->setText("Entry format:");
      idItems << "EMBL (EMBL)" << "SWISS (Swiss-Prot, SpTrEMBL, TrEMBLnew)" << "GENBANK (Genbank, DDBJ)" << "PIR (NBRF)";
   }
   FormatComboBox->addItems(idItems);
   FormatComboBox->setCurrentIndex(1);

   bool tmpBool;
   if (text == "dbiblast") tmpBool = true; else tmpBool = false;
      BlastIndexVersionLabel->setEnabled(tmpBool);
      BlastIndexVersionComboBox->setEnabled(tmpBool);
      UseFASTASourceFileCheckBox->setEnabled(tmpBool);
   if (text.startsWith("dbx")) tmpBool = true; else tmpBool = false;
      SystemGroupBox->setEnabled(!tmpBool);
      IDCheckBox->setEnabled(tmpBool);
   if (text.startsWith("dbx") && text != "dbxreport" && text != "dbxcompress" && text != "dbxuncompress" && text != "dbxstat") tmpBool = true; else tmpBool = false;
      ResourceNameLabel->setEnabled(tmpBool);
      ResourceNameLineEdit->setEnabled(tmpBool);
      CompressCheckBox->setEnabled(tmpBool);
   if (text == "dbxflat" || text == "dbxgcg" || text == "dbxfasta") tmpBool = true; else tmpBool = false;
      LogOutputFileLineEdit->setEnabled(!tmpBool);
      LogOutputFileLabel->setEnabled(!tmpBool);
      LogOutputFilePushButton->setEnabled(!tmpBool);
   if (text == "dbxflat") {
      StatisticsCheckBox->setEnabled(true);
   } else {
      StatisticsCheckBox->setEnabled(false);
   }
   if (text == "dbxobo" || text == "dbxedam" || text == "dbxresource") tmpBool = true; else tmpBool = false;
      StandardDbNameLineEdit->setEnabled(tmpBool);
   if (text == "dbxfasta" || text == "dbxflat" || text == "dbxgcg" || text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") tmpBool = true; else tmpBool = false;
      SVAndGICheckBox->setEnabled(tmpBool);
   if (text == "dbxobo" || text == "dbxedam" || text == "dbxresource" || text == "dbxtax" || text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") tmpBool = true; else tmpBool = false;
      FormatComboBox->setEnabled(!tmpBool);
      FormatLabel->setEnabled(!tmpBool);
   if (text == "dbxobo" || text == "dbxedam" || text == "dbxresource" || text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") tmpBool = true; else tmpBool = false;
      NamespaceCheckBox->setEnabled(tmpBool);
   if (text == "dbxobo" || text == "dbxedam" || text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") tmpBool = true; else tmpBool = false;
      NameCheckBox->setEnabled(tmpBool);
      ParentCheckBox->setEnabled(tmpBool);
   if (text == "dbxedam" || text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") tmpBool = true; else tmpBool = false;
      HasInputCheckBox->setEnabled(tmpBool);
      HasOutputCheckBox->setEnabled(tmpBool);
      IsIdentifierOfCheckBox->setEnabled(tmpBool);
      IsFormatOfCheckBox->setEnabled(tmpBool);
   if (text == "dbiflat" || text == "dbigcg" || text == "dbxflat" || text == "dbxgcg" || text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") tmpBool = true; else tmpBool = false;
      KeywordsCheckBox->setEnabled(tmpBool);
      TaxonomyCheckBox->setEnabled(tmpBool);
   if (text == "dbxresource" || text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") tmpBool = true; else tmpBool = false;
      URLCheckBox->setEnabled(tmpBool);
      CategoryNameCheckBox->setEnabled(tmpBool);
      TaxonIDCheckBox->setEnabled(tmpBool);
      EdatCheckBox->setEnabled(tmpBool);
      EfmtCheckBox->setEnabled(tmpBool);
      EidCheckBox->setEnabled(tmpBool);
      EtpcCheckBox->setEnabled(tmpBool);
      XrefCheckBox->setEnabled(tmpBool);
      QoutCheckBox->setEnabled(tmpBool);
      QfmtCheckBox->setEnabled(tmpBool);
      QinCheckBox->setEnabled(tmpBool);
      QurlCheckBox->setEnabled(tmpBool);
      RestCheckBox->setEnabled(tmpBool);
      SOAPCheckBox->setEnabled(tmpBool);
   if (text == "dbxtax") tmpBool = true; else tmpBool = false;
      DescriptionCheckBox->setEnabled(!tmpBool);
   if (text == "dbxtax" || text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") tmpBool = true; else tmpBool = false;
      TaxCheckBox->setEnabled(tmpBool);
      RnkCheckBox->setEnabled(tmpBool);
      UpCheckBox->setEnabled(tmpBool);
      GcCheckBox->setEnabled(tmpBool);
      MgcCheckBox->setEnabled(tmpBool);
      DatabaseFileLabel->setEnabled(!tmpBool);
      DatabaseFilenameLineEdit->setEnabled(!tmpBool);
      WildcardFilenamesToExcludeLabel->setEnabled(!tmpBool);
      WildcardFilenamesToExcludeLineEdit->setEnabled(!tmpBool);
   if (text == "dbxreport" || text == "dbxcompress" || text == "dbxuncompress" || text == "dbxstat") {
      tmpBool = true;
      IndexDirectoryLineEdit->setToolTip("indexdir");
   } else {
      tmpBool = false;
      IndexDirectoryLineEdit->setToolTip("indexoutdir");
   }
      DatabaseDirectoryLabel->setEnabled(!tmpBool);
      DatabaseDirectoryLineEdit->setEnabled(!tmpBool);
      DatabaseDirectoryPushButton->setEnabled(!tmpBool);
      ReleaseLineEdit->setEnabled(!tmpBool);
      ReleaseNumberLabel->setEnabled(!tmpBool);
      IndexDateLineEdit->setEnabled(!tmpBool);
      IndexDateLabel->setEnabled(!tmpBool);
   if (text == "dbxreport") tmpBool = true; else tmpBool = false;
      FullreportCheckBox->setEnabled(tmpBool);
   if (text == "dbxstat") tmpBool = true; else tmpBool = false;
      MinOccurencesSpinBox->setEnabled(tmpBool);
      MinOccurencesLabel->setEnabled(tmpBool);
      MaxOccurencesSpinBox->setEnabled(tmpBool);
      MaxOccurencesLabel->setEnabled(tmpBool);
}

void dbDlg::on_IndexDatabasePushButton_clicked()
{
   runEmbossApp->programConfiguration.clear();
   runEmbossApp->programConfiguration.append("-auto");

   if (DatabaseNameLineEdit->isEnabled() && !DatabaseNameLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-dbname="+DatabaseNameLineEdit->text());
   if (DatabaseDirectoryLineEdit->isEnabled() && !DatabaseDirectoryLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-directory="+DatabaseDirectoryLineEdit->text());
   if (DatabaseFilenameLineEdit->isEnabled() && !DatabaseFilenameLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-filenames="+DatabaseFilenameLineEdit->text());
   if (ReleaseLineEdit->isEnabled() && !ReleaseLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-release="+ReleaseLineEdit->text());
   if (IndexDateLineEdit->isEnabled() && !IndexDateLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-date="+IndexDateLineEdit->text());
   if (WildcardFilenamesToExcludeLineEdit->isEnabled() && !WildcardFilenamesToExcludeLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-exclude="+WildcardFilenamesToExcludeLineEdit->text());

   QString fieldsString = "";
   if (IDCheckBox->isEnabled() && IDCheckBox->isChecked()) fieldsString += "id,";
   if (AccNoCheckBox->isChecked()) fieldsString += "acc,";
   if (SVAndGICheckBox->isChecked()) fieldsString += "sv,";
   if (DescriptionCheckBox->isChecked()) fieldsString += "des,";
   if (KeywordsCheckBox->isEnabled() && KeywordsCheckBox->isChecked()) fieldsString += "key,";
   if (TaxonomyCheckBox->isEnabled() && TaxonomyCheckBox->isChecked()) fieldsString += "org,";
   if (NameCheckBox->isEnabled() && NameCheckBox->isChecked()) fieldsString += "nam,";
   if (ParentCheckBox->isEnabled() && ParentCheckBox->isChecked()) fieldsString += "isa,";
   if (NamespaceCheckBox->isEnabled() && NamespaceCheckBox->isChecked()) fieldsString += "ns,";
   if (HasInputCheckBox->isEnabled() && HasInputCheckBox->isChecked()) fieldsString += "hasin,";
   if (HasOutputCheckBox->isEnabled() && HasOutputCheckBox->isChecked()) fieldsString += "hasout,";
   if (IsIdentifierOfCheckBox->isEnabled() && IsIdentifierOfCheckBox->isChecked()) fieldsString += "isid,";
   if (IsFormatOfCheckBox->isEnabled() && IsFormatOfCheckBox->isChecked()) fieldsString += "isfmt,";
   if (URLCheckBox->isEnabled() && URLCheckBox->isChecked()) fieldsString += "url,";
   if (CategoryNameCheckBox->isEnabled() && CategoryNameCheckBox->isChecked()) fieldsString += "cat,";
   if (TaxonIDCheckBox->isEnabled() && TaxonIDCheckBox->isChecked()) fieldsString += "taxid,";
   if (EdatCheckBox->isEnabled() && EdatCheckBox->isChecked()) fieldsString += "edat,";
   if (EfmtCheckBox->isEnabled() && EfmtCheckBox->isChecked()) fieldsString += "efmt,";
   if (EidCheckBox->isEnabled() && EidCheckBox->isChecked()) fieldsString += "eid,";
   if (EtpcCheckBox->isEnabled() && EtpcCheckBox->isChecked()) fieldsString += "etpc,";
   if (XrefCheckBox->isEnabled() && XrefCheckBox->isChecked()) fieldsString += "xref,";
   if (QoutCheckBox->isEnabled() && QoutCheckBox->isChecked()) fieldsString += "qout,";
   if (QfmtCheckBox->isEnabled() && QfmtCheckBox->isChecked()) fieldsString += "qfmt,";
   if (QinCheckBox->isEnabled() && QinCheckBox->isChecked()) fieldsString += "qin,";
   if (QurlCheckBox->isEnabled() && QurlCheckBox->isChecked()) fieldsString += "qurl,";
   if (RestCheckBox->isEnabled() && RestCheckBox->isChecked()) fieldsString += "rest,";
   if (SOAPCheckBox->isEnabled() && SOAPCheckBox->isChecked()) fieldsString += "soap,";
   if (TaxCheckBox->isEnabled() && TaxCheckBox->isChecked()) fieldsString += "tax,";
   if (RnkCheckBox->isEnabled() && RnkCheckBox->isChecked()) fieldsString += "rnk,";
   if (UpCheckBox->isEnabled() && UpCheckBox->isChecked()) fieldsString += "up,";
   if (GcCheckBox->isEnabled() && GcCheckBox->isChecked()) fieldsString += "gc,";
   if (MgcCheckBox->isEnabled() && MgcCheckBox->isChecked()) fieldsString += "mgc,";
   if (!fieldsString.isEmpty()) {
      fieldsString.chop(1); // remove the last comma
      runEmbossApp->programConfiguration.append("-fields="+fieldsString);
   }
   if (StatisticsCheckBox->isEnabled() && StatisticsCheckBox->isChecked()) runEmbossApp->programConfiguration.append("-statistics");
   if (CompressCheckBox->isEnabled() && !CompressCheckBox->isChecked()) runEmbossApp->programConfiguration.append("-nocompressed");
   if (LogOutputFileLineEdit->isEnabled() && !LogOutputFileLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-outfile="+LogOutputFileLineEdit->text());
   if (SystemGroupBox->isEnabled() && MaximumIndexLengthSpinBox->value() != 0) runEmbossApp->programConfiguration.append("-maxindex="+MaximumIndexLengthSpinBox->text());
   if (SystemGroupBox->isEnabled() && !SortOptionsLineEdit->text().isEmpty() && SortOptionsLineEdit->text() != "-T . -k 1,1") runEmbossApp->programConfiguration.append("-sortoptions=\""+SortOptionsLineEdit->text()+"\"");
   if (SystemGroupBox->isEnabled() && !UseSystemSortUtilityCheckBox->isChecked()) runEmbossApp->programConfiguration.append("-nosystemsort");
   if (SystemGroupBox->isEnabled() && !CleanUpTemporaryFilesCheckBox->isChecked()) runEmbossApp->programConfiguration.append("-nocleanup");
   if (ResourceNameLineEdit->isEnabled() && !ResourceNameLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-dbresource="+ResourceNameLineEdit->text());
   if (StandardDbNameLineEdit->isEnabled() && !StandardDbNameLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-standardname="+StandardDbNameLineEdit->text());
   if (MinOccurencesSpinBox->isEnabled()) runEmbossApp->programConfiguration.append("-minimum="+MinOccurencesSpinBox->text());
   if (MaxOccurencesSpinBox->isEnabled()) runEmbossApp->programConfiguration.append("-maximum="+MaxOccurencesSpinBox->text());
   // Program specific configuration
   if (DatabaseTypeComboBox->currentText() == "dbiblast") {
      runEmbossApp->programConfiguration.append("-seqtype="+FormatComboBox->currentText().section(" ", 0, 0));
      runEmbossApp->programConfiguration.append("-blastversion="+BlastIndexVersionComboBox->currentText().section(" ", 0, 0));
      if (UseFASTASourceFileCheckBox->isChecked()) runEmbossApp->programConfiguration.append("-sourcefile");
   } else {
      if (FormatComboBox->isEnabled()) runEmbossApp->programConfiguration.append("-idformat="+FormatComboBox->currentText().section(" ", 0, 0));
   }
   if (DatabaseTypeComboBox->currentText() == "dbxreport" || DatabaseTypeComboBox->currentText() == "dbxcompress" || DatabaseTypeComboBox->currentText() == "dbxuncompress" || DatabaseTypeComboBox->currentText() == "dbxstat") {
      if (!IndexDirectoryLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-indexdir="+IndexDirectoryLineEdit->text());
   } else {
      if (IndexDirectoryLineEdit->isEnabled() && !IndexDirectoryLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-indexoutdir="+IndexDirectoryLineEdit->text());
   }
   runEmbossApp->setEmbossApp(DatabaseTypeComboBox->currentText());
   runEmbossApp->runApp();
   getAvailableDatabases();
}

void dbDlg::on_ProgramComboBox_currentIndexChanged(int index)
{
   if (index == 0) {
      SequencesourcesonlyCheckBox->setEnabled(true);
      HostLineEdit->setEnabled(true); HostLabel->setEnabled(true);
      PathLineEdit->setEnabled(true); PathLabel->setEnabled(true);
      PortSpinBox->setEnabled(true); PortLabel->setEnabled(true);
   } else if (index == 1 || index == 2 || index == 3) {
      SequencesourcesonlyCheckBox->setEnabled(false);
      HostLineEdit->setEnabled(false); HostLabel->setEnabled(false);
      PathLineEdit->setEnabled(false); PathLabel->setEnabled(false);
      PortSpinBox->setEnabled(false); PortLabel->setEnabled(false);
   }
   if (index == 0 || index == 1) {
      ServernameLineEdit->setEnabled(true); ServernameLabel->setEnabled(true);
   } else if (index == 2 || index == 3) {
      ServernameLineEdit->setEnabled(false); ServernameLabel->setEnabled(false);
   }
}

void dbDlg::on_GenerateCachefilePushButton_clicked()
{
   runEmbossApp->programConfiguration.clear();
   runEmbossApp->programConfiguration.append("-auto");

   if (CachefileLineEdit->isEnabled() && !CachefileLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-cachefile="+CachefileLineEdit->text());
   if (SequencesourcesonlyCheckBox->isEnabled() && SequencesourcesonlyCheckBox->isChecked()) runEmbossApp->programConfiguration.append("-sequencesourcesonly");
   if (OutfileLineEdit->isEnabled() && !OutfileLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-outfile="+OutfileLineEdit->text());
   if (HostLineEdit->isEnabled() && !HostLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-host="+HostLineEdit->text());
   if (PathLineEdit->isEnabled() && !PathLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-path="+PathLineEdit->text());
   if (PortSpinBox->isEnabled()) runEmbossApp->programConfiguration.append("-port="+PortSpinBox->text());
   if (ServernameLineEdit->isEnabled() && !ServernameLineEdit->text().isEmpty()) runEmbossApp->programConfiguration.append("-servername="+ServernameLineEdit->text());

   QString app = ProgramComboBox->currentText().section("(", 1, 1);
   app.chop(1);
   runEmbossApp->setEmbossApp(app);
   runEmbossApp->runApp();
   getAvailableServer();
}

void dbDlg::on_DatabasePushButton_clicked()
{
    DbtellTextEdit->document()->clear();
    QProcess dbtell;
    QString prog = ksettings->embossBin()+"dbtell"+
       #ifdef Q_WS_WIN
       ".exe"+
       #endif
       " "+DatabaseLineEdit->text();
    if (DbtellFullCheckBox->checkState()) prog += " -full";
    dbtell.start(prog);
    if (!dbtell.waitForFinished(ksettings->maxExeTime())) return; // waits for program dbtell to finish
    QByteArray programOutput = dbtell.readAll();
    DbtellTextEdit->setText(QString(programOutput));
}

void dbDlg::on_ServerPushButton_clicked()
{
    ServertellTextEdit->document()->clear();
    QProcess servertell;
    QString prog = ksettings->embossBin()+"servertell"+
       #ifdef Q_WS_WIN
       ".exe"+
       #endif
       " "+ServerLineEdit->text();
    if (ServertellFullCheckBox->checkState()) prog += " -full";
    servertell.start(prog);
    if (!servertell.waitForFinished(ksettings->maxExeTime())) return; // waits for program servertell to finish
    QByteArray programOutput = servertell.readAll();
    ServertellTextEdit->setText(QString(programOutput));
}
