/***************************************************************************
 *   Copyright (C) 2007-2010 by Tobias Schwarz                             *
 *   mail@tobias-schwarz.net                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include "about.h"
#include "kemboss.h"

kembossWindow::kembossWindow(QMainWindow *parent) : QMainWindow(parent)
{
   setWindowIcon(QIcon(":/new/prefix1/kemboss.png"));
   fileToolBar = NULL;
   editToolBar = NULL;
   viewToolBar = NULL;
   graphToolBar = NULL;
   runToolBar = NULL;
   helpToolBar = NULL;
   kpwOutputGraphicsView = NULL;
}

void kembossWindow::createActions()
{
   viewFileToolbarAct = new QAction(tr("File"), this);
   viewFileToolbarAct->setStatusTip(tr("View file toolbar"));
   viewFileToolbarAct->setCheckable(true);
   viewFileToolbarAct->setChecked(true);
   connect(viewFileToolbarAct, SIGNAL(triggered()), this, SLOT(viewFileToolbar()));

   viewEditToolbarAct = new QAction(tr("Edit"), this);
   viewEditToolbarAct->setStatusTip(tr("View edit toolbar"));
   viewEditToolbarAct->setCheckable(true);
   viewEditToolbarAct->setChecked(true);
   connect(viewEditToolbarAct, SIGNAL(triggered()), this, SLOT(viewEditToolbar()));

   viewViewToolbarAct = new QAction(tr("View"), this);
   viewViewToolbarAct->setStatusTip(tr("View view toolbar"));
   viewViewToolbarAct->setCheckable(true);
   viewViewToolbarAct->setChecked(true);
   connect(viewViewToolbarAct, SIGNAL(triggered()), this, SLOT(viewViewToolbar()));

   viewGraphToolbarAct = new QAction(tr("Graph"), this);
   viewGraphToolbarAct->setStatusTip(tr("View graph toolbar"));
   viewGraphToolbarAct->setCheckable(true);
   viewGraphToolbarAct->setChecked(true);
   connect(viewGraphToolbarAct, SIGNAL(triggered()), this, SLOT(viewGraphToolbar()));

   viewRunToolbarAct = new QAction(tr("Run"), this);
   viewRunToolbarAct->setStatusTip(tr("View run toolbar"));
   viewRunToolbarAct->setCheckable(true);
   viewRunToolbarAct->setChecked(true);
   connect(viewRunToolbarAct, SIGNAL(triggered()), this, SLOT(viewRunToolbar()));

   viewHelpToolbarAct = new QAction(tr("Help"), this);
   viewHelpToolbarAct->setStatusTip(tr("View help toolbar"));
   viewHelpToolbarAct->setCheckable(true);
   viewHelpToolbarAct->setChecked(true);
   connect(viewHelpToolbarAct, SIGNAL(triggered()), this, SLOT(viewHelpToolbar()));

   reportBugAct = new QAction(QIcon(":/new/prefix1/mail.png"), tr("&Report GUI Bug"), this);
   reportBugAct->setStatusTip(tr("Report a GUI bug"));
   connect(reportBugAct, SIGNAL(triggered()), this, SLOT(reportBug()));

   aboutAct = new QAction(QIcon(":/new/prefix1/about.png"), tr("About kemboss"), this);
   aboutAct->setStatusTip(tr("About kemboss"));
   connect(aboutAct, SIGNAL(triggered()), this, SLOT(aboutKemboss()));
}

void kembossWindow::readSettings()
{
   QSettings settings(QSettings::UserScope, "kemboss.org", objectName());
   QPoint pos = settings.value("pos").toPoint();
   QSize size = settings.value("size").toSize();
   if (!size.isEmpty()) resize(size);
   if (!pos.isNull()) move(pos);
   // Read toolbar status
   if (fileToolBar != NULL) showFileToolbar(settings.value("show_file_toolbar", true).toBool());
   if (editToolBar != NULL) showEditToolbar(settings.value("show_edit_toolbar", true).toBool());
   if (viewToolBar != NULL) showViewToolbar(settings.value("show_view_toolbar", true).toBool());
   if (graphToolBar != NULL) {
      if (kpwOutputGraphicsView != NULL) showGraphToolbar(settings.value("show_graph_toolbar", true).toBool());
      else showGraphToolbar(settings.value("show_graph_toolbar", false).toBool());
   }
   if (runToolBar != NULL) showRunToolbar(settings.value("show_run_toolbar", true).toBool());
   if (helpToolBar != NULL) showHelpToolbar(settings.value("show_help_toolbar", true).toBool());
   highlightInput = settings.value("highlight_input").toInt();
   highlightOutput = settings.value("highlight_output").toInt();
}

void kembossWindow::writeSettings()
{
   QSettings settings(QSettings::UserScope, "kemboss.org", objectName());
   settings.setValue("pos", pos());
   settings.setValue("size", size());
   // Write toolbar status
   if (fileToolBar != NULL) settings.setValue("show_file_toolbar", fileToolBar->isVisible());
   if (editToolBar != NULL) settings.setValue("show_edit_toolbar", editToolBar->isVisible());
   if (viewToolBar != NULL) settings.setValue("show_view_toolbar", viewToolBar->isVisible());
   if (graphToolBar != NULL) settings.setValue("show_graph_toolbar", graphToolBar->isVisible());
   if (runToolBar != NULL) settings.setValue("show_run_toolbar", runToolBar->isVisible());
   if (helpToolBar != NULL) settings.setValue("show_help_toolbar", helpToolBar->isVisible());
   if (!curFile.isEmpty()) settings.setValue("last_settings", curFile);
   settings.setValue("highlight_input", highlightInput);
   settings.setValue("highlight_output", highlightOutput);
}

void kembossWindow::open()
{
   if (maybeSave()) {
      QString fileName = QFileDialog::getOpenFileName(this, tr("Open settings"),qobject_cast<kemboss *>(kembossParent)->ksettings.currentDir(),objectName()+tr(" files (*.")+objectName()+tr(");;All files (*)"));
      if (!fileName.isEmpty())
         if (loadFile(fileName))
            setCurrentFile(fileName);
   }
}

bool kembossWindow::save()
{
   if (curFile.isEmpty()) {
      return saveAs();
   } else {
      return saveFile(curFile);
   }
}

bool kembossWindow::saveAs()
{
   QString fileName = QFileDialog::getSaveFileName(this, tr("Save settings"),qobject_cast<kemboss *>(kembossParent)->ksettings.currentDir(),objectName()+tr(" files (*.")+objectName()+tr(");;All files (*)"));
   if (fileName.isEmpty())
      return false;
   if (!fileName.endsWith("."+objectName()))
      fileName.append("."+objectName());
   return saveFile(fileName);
}

void kembossWindow::createRecentFiles()
{
   for (int i = 0; i < MaxRecentFiles; ++i) {
      recentFileActs[i] = new QAction(this);
      recentFileActs[i]->setVisible(false);
      connect(recentFileActs[i], SIGNAL(triggered()), this, SLOT(openRecentFile()));
      recentSettingsMenu->addAction(recentFileActs[i]);
    }
    updateRecentFileActions();
}

void kembossWindow::updateRecentFileActions()
{
   QSettings settings(QSettings::UserScope, "kemboss.org", objectName());
   QStringList files = settings.value("recentFileList").toStringList();
   int numRecentFiles = qMin(files.size(), (int)MaxRecentFiles);

   for (int i = 0; i < numRecentFiles; ++i) {
      QString text = tr("&%1 %2").arg(i + 1).arg(strippedName(files[i]));
      recentFileActs[i]->setText(text);
      recentFileActs[i]->setData(files[i]);
      recentFileActs[i]->setVisible(true);
   }
   for (int j = numRecentFiles; j < MaxRecentFiles; ++j)
      recentFileActs[j]->setVisible(false);
}

void kembossWindow::openRecentFile()
{
   QAction *action = qobject_cast<QAction *>(sender());
   if (action)
      if (maybeSave())
         if (loadFile(action->data().toString()))
            setCurrentFile(action->data().toString());
}

void kembossWindow::assistant(const QString &page)
{
    if (qobject_cast<kemboss *>(kembossParent)->ksettings.useOnlineDoc()) {
        QString progName = page;
        progName.replace(0,1,page[0].toUpper());
        QDesktopServices::openUrl("http://emboss.open-bio.org/wiki/Appdoc:"+progName);
    } else {
        if (QFile::exists(qobject_cast<kemboss *>(kembossParent)->ksettings.embossHelp()+page+".html"))
            QDesktopServices::openUrl(QUrl::fromLocalFile(qobject_cast<kemboss *>(kembossParent)->ksettings.embossHelp()+page+".html"));
        else if (QFile::exists(qobject_cast<kemboss *>(kembossParent)->ksettings.embossHelp()+"../../html/embassy/phylipnew/"+page+".html"))
            QDesktopServices::openUrl(QUrl::fromLocalFile(qobject_cast<kemboss *>(kembossParent)->ksettings.embossHelp()+"../../html/embassy/phylipnew/"+page+".html"));
    }
}

void kembossWindow::aboutKemboss()
{
   aboutForm *about = new aboutForm(qobject_cast<kemboss *>(kembossParent));
   about->show();
}
